<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="cfg">
    <xsl:output method="xml" omit-xml-declaration="yes"/>
    <xsl:variable name="VERSION">12</xsl:variable>

    <xsl:template match="cfg:cache[@type='IndexedFileCache']/cfg:parameter/@name[.='indexDepth']">
        <xsl:attribute name="name">index_depth</xsl:attribute>
    </xsl:template>

    <xsl:template match="cfg:processor[@type='RenameProcessor' and cfg:parameter[@name='node_name']/@eval]/@type">
        <xsl:attribute name="type">RenameWithXPathProcessor</xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:rename[@prefix and not(@name)]">
        <processor type="RemoveNamespaceProcessor" match="{@match}"/>
    </xsl:template>
    <xsl:template match="cfg:processors/cfg:*/@prefix | cfg:processors/cfg:*[not(self::cfg:processor)]/cfg:*/@prefix"/>
    <xsl:template match="cfg:processors/cfg:*[@prefix!='']/@name">
        <xsl:attribute name="name">
            <xsl:for-each select="parent::cfg:*">
                <xsl:value-of select="@prefix"/>:<xsl:value-of select="@name"/>
            </xsl:for-each>
        </xsl:attribute>
    </xsl:template>

    <xsl:template match="cfg:processor[@type='MoveProcessor' or @type='CopyProcessor']
                        /cfg:parameter[@name='destination_axis' and text()='following']">
        <xsl:message terminate="yes">
            Axis 'following' no more supported in MoveProcessor and CopyProcessor.
            Please use axis 'following-sibling' or 'ancestor' instead.
        </xsl:message>
    </xsl:template>
    <xsl:template match="cfg:to-following[@sibling='false' or not(@sibling)]">
        <xsl:message terminate="yes">
            Axis 'to-following' no more supported in <xsl:value-of select="local-name(parent::cfg:*)"/>.
            Please set attribute @sibling to 'true' or use element to-ancestor instead.
        </xsl:message>
    </xsl:template>
    <xsl:template match="cfg:to-following[@sibling='true']">
        <to-following-sibling>
            <xsl:apply-templates select="@* | * | text() | comment()"/>
        </to-following-sibling>
    </xsl:template>
    <xsl:template match="cfg:to-following/@sibling[.='true']"/>

    <xsl:template match="cfg:processor/@generate-match"/>
    <xsl:template match="cfg:processor[@generate-match]/@match">
        <xsl:attribute name="match">eval(<xsl:value-of select="@generate-match"/>)</xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:view/@eval-authorized | cfg:view-template/@eval-authorized">
        <xsl:attribute name="authorized"><xsl:value-of select="."/></xsl:attribute>
    </xsl:template>

    <xsl:template match="cfg:select/cfg:if-descendant">
        <xsl:message terminate="yes">
            Element &lt;if-descendant&gt; is no more supported, please use processor SelectAncestorProcessor instead.
        </xsl:message>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='SelectIfDescendantProcessor']">
        <xsl:copy>
            <xsl:attribute name="type">SelectAncestorProcessor</xsl:attribute>
            <xsl:attribute name="match">
                <xsl:value-of select="."/>
                <xsl:text>/descendant::</xsl:text>
                <xsl:for-each select="cfg:parameter[@name='if_descendant_prefix']"><xsl:value-of select="text()"/>:</xsl:for-each>
                <xsl:for-each select="cfg:parameter[@name='if_descendant_name']"><xsl:value-of select="text()"/></xsl:for-each>
                <xsl:if test="not(cfg:parameter[@name='if_descendant_name'])">*</xsl:if>
                <xsl:for-each select="cfg:parameter[@name='if_descendant_predicate']">[<xsl:value-of select="text()"/>]</xsl:for-each>
            </xsl:attribute>
            <xsl:apply-templates select="cfg:parameter[not(starts-with(@name, 'if_descendant_'))]"/>
            <parameter name="ancestor" eval="{@match}"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='MoveProcessor' or @type='CopyProcessor']/@match">
        <xsl:attribute name="match">
            <xsl:variable name="match">
                <xsl:call-template name="REMOVE_PREDICATE">
                    <xsl:with-param name="path" select="."/>
                </xsl:call-template>
            </xsl:variable>
            <xsl:value-of select="."/>
            <xsl:if test="not(contains($match,'/@') or contains($match,'/attribute::')
                           or contains($match,'/text()') or contains($match,'/comment()'))">[* or not(*)]</xsl:if>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='AppendAggregateProcessor']">
        <xsl:copy>
            <xsl:copy-of select="@type"/>
            <xsl:attribute name="match">
                <xsl:value-of select="."/> | <xsl:value-of select="."/>/<xsl:value-of
                    select="cfg:parameter[@name='values']/@eval"/>
            </xsl:attribute>
            <xsl:apply-templates select="cfg:parameter[not(@name='values')]"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='SelectGroupByProcessor']">
        <xsl:if test="cfg:parameter[@name='context_depth']/text() > 1">
            <xsl:message terminate="yes">Please modify SelectGroupByProcessor with context_depth&gt;1</xsl:message>
        </xsl:if>
        <xsl:copy>
            <xsl:copy-of select="@type"/>
            <xsl:attribute name="match">/* | <xsl:value-of select="@match"/>/<xsl:value-of
                    select="cfg:parameter[@name='group_by']/@eval"/></xsl:attribute>
            <xsl:apply-templates select="cfg:parameter[not(@name='group_by' or @name='context_depth')]"/>
            <parameter name="ancestor">
                <xsl:attribute name="eval">
                    <xsl:text>.</xsl:text>
                    <xsl:call-template name="REVERT_PATH">
                        <xsl:with-param name="path">
                            <xsl:call-template name="REMOVE_PREDICATE">
                                <xsl:with-param name="path" select="cfg:parameter[@name='group_by']/@eval"/>
                            </xsl:call-template>
                        </xsl:with-param>
                    </xsl:call-template>
                    <xsl:text>/parent::*</xsl:text>
                </xsl:attribute>
            </parameter>
        </xsl:copy>
    </xsl:template>

    <xsl:template name="REMOVE_PREDICATE">
        <xsl:param name="path"/>
        <xsl:choose>
            <xsl:when test="contains($path, '[')">
                <xsl:call-template name="REMOVE_PREDICATE">
                    <xsl:with-param name="path" select="concat(substring-before($path,'['), substring-after($path,']'))"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$path"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="REVERT_PATH">
        <xsl:param name="path"/>
        <xsl:if test="contains($path, '/')">
            <xsl:call-template name="REVERT_PATH">
                <xsl:with-param name="path" select="substring-after($path, '/')"/>
            </xsl:call-template>/parent::<xsl:value-of select="substring-before($path, '/')"/>
        </xsl:if>
    </xsl:template>

    <!-- common -->
    <xsl:template match="/cfg:config/@version">
        <xsl:attribute name="version">
            <xsl:value-of select="$VERSION"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@* | comment()" priority="-1">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*" priority="-1">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>